
/*
PROJECT : tinyTablebot
AUTHOR  : SANCHIT MALHOTRA
MCU     : Attiny2313
F_CPU   : 4000000 (4MHz)

Attiny2313 full datasheet- http://www.atmel.com/images/doc2543.pdf

Atmel app. note on PWM duty cycle measurement- http://www.atmel.com/images/doc8014.pdf

Motor driver: L293d

L293d pins            |  Attiny2313 pins

Motor 1 enable pin    :   PB0
Motor 2 enable pin    :   PB1
Motor 1 direction pin :   PB2
Motor 2 direction pin :   PB3

Red LED               :   PD5
Green LED             :   PD3
*/

#include<avr/io.h>
#include<avr/interrupt.h>
#include<util/delay.h>

volatile unsigned int width = 0;
volatile unsigned int left;
volatile unsigned int right;
uint8_t rising = 0;

void check(void);
void forward(void);
void turnleft(void);
void turnright(void);
void stop(void);


ISR(TIMER1_CAPT_vect)       // timer capture interrupt
{
  if(rising==1)
  {
    TCCR1B &= ~(1<<ICES1); // falling edge triggers the interrupt from here on
    TCNT1 = 0;              // set timer1 to 0
    rising = 0;
  }
  else
  {
    width = ICR1;           // ICR1 captures the 'image' of the TCNT1 at interrupt  
	TCCR1B |= (1<<ICES1);  // rising edge triggers the interrupt from here on
	rising = 1;	
  }
}


int main(void)
{
  DDRB = 0xff;
  PORTB = 0x00;
  _delay_ms(2000);
  
  DDRD = 0x3C;
  
  TCCR1B |= (1<<CS10);     // initialising timer 1 with prescaler = 1
  
  TCCR1B |= (1<<ICES1);    // rising edge triggers the interrupt                  
  TIMSK |= (1<<ICIE1);     // enable timer capture interrupt  
  
  sei();
  
  while(1)
  {
    if(width > 1600)
	{
	  PORTD &= ~(1<<3);    // turn off green LED
	  PORTD |= (1<<5);     // turn on red LED
	  check();
	  PORTD &= ~(1<<5);    // turn off red LED
	}
	else
	{
	  PORTD |= (1<<3);     // turn on green LED
      forward();
	}
  }
}

void check()
{
  
  stop();           // stop 
  
  turnleft();       // turn left
  
  stop();           // stop
  
  left = width;     // ******* take left distance reading *******
   
  turnright();      // center
  
  turnright();      // turn full right
  
  stop();           // stop
  
  right = width;    // ******* take right distance reading ******
  
  turnleft();       // center  
  
  stop();           // stop
  
  if(left<right)
  {
    turnleft();     // turn left
  }
  else
  {
    turnright();    // turn right
  }
 
  stop();           // stop
}

void forward()
{
  PORTB = 0x0f;     // forward
}

void turnleft()
{
  PORTB = 0x07;     // turn left
  _delay_ms(500);  
}

void turnright()
{
  PORTB = 0x0B;     // turn right
  _delay_ms(500);
}

void stop()
{
  PORTB = 0x00;     // stop for a while
  _delay_ms(200);
}
